\name{tsm}
\alias{tsm}
\title{Treatment-Selection Markers using Potential Outcomes Framework}
\description{
}
\usage{
tsm(formula, data, treatment,subset, na.action, 
model = FALSE, x = TRUE, y = TRUE,est.scale = c("risk","clpo"),
link = c("logit","probit"),start = NULL,parsen = NULL,signif.level = NULL,
constrained = TRUE,fpr = seq(.1,.9,by=.1),q = c(.035,.085),
optimfun = c("optim","nmk","hjk"),control = list(),
nboot = 0,seed = NULL,parallel = c("no", "multicore", "snow"),
ncpus = getOption("boot.ncpus", 1L),cl = NULL,
verbose = FALSE) 
}

\arguments{
\item{formula}{an object of class "formula" (or one that can be coerced to that class): a symbolic description of the model to be fitted.}
\item{data}{an optional data frame, list or environment (or object coercible by as.data.frame to a data frame) 
			containing the variables in the model. If not found in data, the variables are taken from 
			environment(formula), typically the environment from which tsm is called.
            }
\item{treatment}{a logical (F/T) or numeric (0/1) vector of treatment indicators or a right handed formula of the form '~Z' if 
				'Z' denotes the treatment indicator in 'data'. The formula should not contain any other terms 
				such as a '-1'.
                }
\item{subset}{an optional vector specifying a subset of observations to be used in the fitting process.}
\item{na.action}{a function which indicates what should happen when the data contain NAs. The default is set 
				 by the na.action setting of options, and is na.fail if that is unset. The factory-fresh 
				 default is na.omit. Another possible value is NULL, no action. Value na.exclude can be useful.
                }           
\item{model}{logical, whether to return the model frame, defaults to FALSE.}
\item{y}{logical, whether to return the model response, defaults to TRUE.}
\item{x}{logical, whether to return the model matrix '[1,S]' corresponding to markers 'S' and the treatment indicator 'Z', 
		defaults to TRUE.}
\item{link}{a character string specifying 'logit' or 'probit' link is to be used, the argument to \code{make.link}.
			Defaults to 'logit'.}
\item{start}{The starting parameter vector on "risk-scale" whose entries correspond to the model matrix 
            [1,S1,..,Sd,Z,Z*S1,..,Z*Sd] for a d-dimensional marker 'S' and treatment indicator 'Z'. 
            Defaults to NULL, in which case the best (judged by likelihood) of the zero vector and  
            estimates from glm of the models with main effect, main effect and interaction with treatment. 
            }
\item{est.scale}{should estimation occur on scale of parameters in risk model ("risk") or 
in cumulative link model for potential outcomes ("clpo") ? Defaults to "risk" and it is sufficient to provide the first character.}            
\item{constrained}{logical, indicates whether to find the regression parameter based on \code{glm} or 
via passing the constrained objective function to \code{optim} (TRUE), defaults to TRUE.}
\item{parsen}{Parameter for sensitivity analysis, defaults to NULL. If not NULL, it should have 
half the length of start corresponding to model matrix [1,S].}
\item{fpr}{false positive rates 't' at which roc(t) is estimated, defaults to (0.1,..,0.9)}
\item{q}{numeric vector of thresholds of interest for risk difference between treatment arms}
\item{signif.level}{a vector of significance levels for constructing percentile bootstrap based confidence intervals around 
                  parameter estimates, defaults to NULL. For the roc curve, for each entry 'a' in signif.level, 
                  a confidence interval at quantiles (roc(t)_{a/2},roc(t)_{(1-a)/2} is constructed, where 't' is the input 'fpr'
                  to \code{\link{tsm}.}
                  }
\item{optimfun}{a character string specifying the optimizer to be used, defaults to the default method of \code{\link{optim}}, "Nelder-Mead". 
                Optional derivative-free optimizers which have the same interface and output structure may also be used such as "nmk" or "hjk" in package \link{dfoptim}.}                  
\item{control}{the control parameter for 'optimfun' above, defaults to list().}
\item{nboot}{number of bootstrap iterations, defaults to 0.}
\item{seed}{seed to be set prior to calling \link{boot}, defaults to NULL.}
\item{parallel}{please see package \link{boot}.}
\item{ncpus}{please see package \link{boot}.}
\item{cl}{please see package \link{boot}.}
\item{verbose}{if TRUE, value of the objective function at the parameter and the parameter with which optim() calls are printed.}
}

\details{
Let 'Z' denote the binary treatment indicator, 'S' a d-dimensional vector 
of continuous markers (or a prespecified combination score thereof) and denote the function
based on which decision rules are constructed as Q(S): R^\{d\}->R such that higher 
values of Q(S) indicate higher probability of benefitting from treatment. 
In this implementation Q(S) = risk_0 - risk_1 = P(Y=1 | Z=0,S) - P(Y=1 | Z=1,S)
and parameters of interest are functions of Q(S) (i.e. TPF(Q), FPF(Q), CDF(Q), 
ROC(t), AUC and regression parameter 'theta'). 
If 'g' denotes a link function, then the two equivalent parametrizations can be used. 
Theta is the parameter in the risk model:
g\{P[Y(Z) = 1 | S]\} = theta_0 + theta_1'S + theta_2*Z + theta_3'(Z*S). 
Alpha is the parameter in the cumulative link model under monotonicity assumption (MS) and 
it is a one-to-one function of Theta with alpha_1 = (theta_0 + theta_2,theta_1 + theta_3) and
alpha_2 = (theta_0,theta_1). 
Confidence intervals for parameters are based on 
stratified (on Z) percentile bootstrap and can be easily viewed by summary(fit). 
Plots of the ROC-curve, CDF(Q), TPF(Q) and FPF(Q) can be done via plot(fit), 
please see  \code{\link{plot.tsm}}.
'start', the vector of starting values for the initial parameter vector is always given as 'theta', the parameter on risk scale. 
Since est_scale = "risk" by default, the risks Risk(Z,S),Z=0,1 in the objective function are computed as a function of 'theta'. 
If est_scale = "clpo" then 'start' is converted to 'alpha' corresponding to clpo-scale via the function alpha.theta() and 
Risk(Z,S), Z=0,1 are expressed in terms of 'alpha' in the objective function.
}

\value{
An object of class "tsm". The functions coef(), summary() and plot() can be used on the object.
The object is a list which contains the following elements:
\item{coefficients}{estimate of risk-scale regression parameter}
\item{auc}{estimate of auc under MS}
\item{roc}{estimate of ROC curve under MS at input false positive rates 't' based on ROC(t) = TPF.Q(FPF.Q^\{-1\}(t)).}
\item{tpf.q}{estimate of true prositive fraction(s) under MS at input threshold(s) 'q'}
\item{fpf.q}{estimate of false prositive fraction(s) under MS at input threshold(s) 'q'}
\item{Rinvp.q}{estimate of risk difference quantile at input threshold(s) 'q'}
\item{auc_n}{estimate of AUC of the 'benefitted' group vs 'unaffected or harmed' group in sensitivity analysis.}
\item{auc_u}{estimate of AUC of the 'benefitted' group vs the 'unaffected' group in sensitivity analysis.}
\item{auc_h}{estimate of AUC for the 'benefitted' group vs 'harmed' group in sensitivity analysis.}
\item{roc_n}{estimate of ROC curve at input fpr for the 'benefitted' group vs 'unaffected or harmed' group 
in sensitivity analysis.}
\item{roc_u}{estimate of ROC curve at input fpr for the 'benefitted' group vs 'unaffected' group in sensitivity analysis.}
\item{roc_h}{estimate of ROC curve at input fpr for the 'benefitted' group vs 'harmed' group in sensitivity analysis.}
\item{start}{the initial parameter value for optim() on scale of estimation or for glm() on risk scale.}
\item{Start}{list containing the candidates for 'start' above and their corresponding likelihood, unless 'start' had been supplied.} 
\item{theta}{estimate of risk-model parameter as a matrix}
\item{alpha}{estimate of cumulative-link model parameter as a matrix}
\item{Q}{the ordered unique estimates of risk difference Q(S) = Risk(Z=0,S)-Risk(Z=1,S) used in the computation of TPF(Q)'s and FPF(Q)'s. }
\item{TPF.Q}{estimated true positive fraction TPF(Q) under MS, a numeric vector of length(Q).}
\item{FPF.Q}{estimated false positive fraction FPF(Q) under MS, a numeric vector of length(Q).}
\item{roc.fun}{a function that returns a vector of linearly interpolated points on the estimated ROC curve under MS at the input vector.}
\item{tpf.fun}{a function that returns a vector of linearly interpolated points on the estimated TPF(Q) curve under MS at the input vector.}
\item{fpf.fun}{a function that returns a vector of linearly interpolated points on the estimated FPF(Q) curve under MS at the input vector.}
\item{Note}{that quantities corresponding to 'TPF.Q', 'FPF.Q', 'roc.fun', 'tpf.fun', 'fpf.fun', 'roc', 'auc' in sensitivity analysis will be supplemented 
        with '_n', '_u', or '_h' accordingly while "Rinvp.q" is suppressed. }        
\item{fpr}{input false positive rate 't' at which ROC(t) is estimated.}
\item{cov}{bootstrapped estimates of variance of parameters}
\item{ci}{bootstrapped confidence intervals}
\item{q}{input threshold for risk difference Q(S)}
\item{signif.level}{input significance level}
\item{boot}{object returned by \code{boot}}
\item{optim}{object returned by \code{optim} if constrained = TRUE, NULL otherwise.}
\item{link}{result of \code{make.link(link)}}
\item{call}{the matched call}
\item{formula}{the input formula}
\item{treatment}{the input formula for treatment}
\item{na.action}{information returned by \code{model.frame} on the special handling of NAs.}
\item{y}{if requested, the response used.}
\item{x}{if requested, the model matrix used.}
\item{z}{if 'x' was requested, the treatment indicator used.}
\item{na.action}{(where relevant) information returned by model.frame on the special handling of NAs.}
}


\note{}
\references{
  \itemize{
    \item
      Author: Ying Huang, Peter B. Gilbert and Holly Janes
      \emph{Assessing Treatment-Selection Markers using a Potential Outcomes Framework}\cr
      \url{http://labs.fhcrc.org/huang/Research/index.html}    
  }
}
\author{
  Ying Huang \email{yhuang@fhcrc.org},
  Krisztian Sebestyen \email{ksebestyen@gmail.com}\cr
}

\seealso{
  \code{\link{plot.tsm}} \code{\link{summary.tsm}}
}

\examples{
library(tsm)
library(msm)
n<-700 ## sample size for the randomized trial
theta<-c(1.746,-1.0,-0.236,-0.2)  ### risk model parameter
fpr<-seq(0.1,0.9,0.1)  ### points of interest on the ROC curve
q<-c(0.035,0.085,.1,.15)    ### thresholds for risk difference
Z<-c(rep(0,n/2),rep(1,n/2))  ### treatment assignment
set.seed(560)
S<-rtnorm(n,mean=3,sd=0.5)
XX<-cbind(1,S,Z,S*Z)
Y<-rbinom(n,1,expit(tcrossprod(XX,t(theta))))
data <- data.frame(Y,Z,S)
mean(Y[Z==1]);mean(Y[Z==0])

gfit <-glm(Y~S*Z,data,family = 'binomial');coef(gfit)
set.seed(1);fit <-tsm(Y~S,data,~Z,fpr = fpr,q = q,control = list(maxit = 1e5),nboot = 30) #with confidence interval
coef(fit);
summary(fit,signif.level = .05)
str(fit);
plot(fit);# plot roc curve 
plot(fit,"roc",signif.level = 0.08) # with confidence interval
plot(fit,c("t","f","c")) #plot tpf(q),fpf(q),cdf_Q

gfit<-tsm(Y~S,data,~Z,fpr = fpr,q = q,constrained = F); #glm estimate

################################################################################################
#sensitivity analysis for a single marker - a numerical example
################################################################################################
# warn = -1: turns off warning
# warn =  0: 10 errors stored/printed after function returns, use warnings() to see if more 
# warn =  1: error printed as they occur
# warn =  2: raises error instead of warning
# see 'options' for more details
# options(warn = -1) 

parsen <- c(0.5,0)
fit.r <-try(tsm(Y~S,data,~Z,est.scale = "c",parsen = parsen,start = theta,fpr = fpr,q = q))
summary(fit.r)
plot(fit.r,c('n','u','h'))

}
\keyword{auc}
\keyword{roc}
\keyword{area}
\keyword{receiver}










